<?php

/**
 * Class that permits to load the Clients Configuration file
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 */
class ClientConfiguration {

    protected $clients;

    public function __construct($dir = "") {
        if ($dir == '') {
            $file = dirname(dirname(__FILE__)) . "/config/clientKeys.xml";
        } else {
            $file = $dir . "clientKeys.xml";
        }
        $this->loadClients($file);
    }

    public function getSecret($client_id) {
        return $this->clients[$client_id]["Key"];
    }

    public function isClient($client_id) {
        return (array_key_exists($client_id, $this->clients));
    }

    public function getAllowedScopes($client_id) {
        return $this->clients[$client_id]["Scopes"];
    }

    public function getGenerateAccessTokenMethod($client_id) {
        return $this->clients[$client_id]['GenerateAccessTokenMethod'];
    }

    private function loadClients($file) {
        $xml = simplexml_load_file($file);
        if (strcmp($xml->getName(), "Clients") == 0) {
            $arr = $xml->children();
            $aux = array();
            foreach ($arr as $child) {
                $id = (string) $child['id'];
                $tokenLifetime = (int) $child->TokenLifetime;
                $key = (string) $child->Key;
                $gatm = (string) $child->GenerateAccessTokenMethod;
                $scopes = array();
                $var = 0;
                foreach ($child->AllowedScopes->children() as $scope) {
                    $scope_id = (string) $scope['id'];
                    $scopes[$var] = $scope_id;
                    $var++;
                }
                $aux[$id] = array("TokenLifetime" => $tokenLifetime, "Key" => $key, "Scopes" => $scopes, "GenerateAccessTokenMethod" => $gatm);
            }
        } else {
            header("HTTP/1.0 400 Bad Request");
            header("Content-Type: application/json");
            header("Cache-control:no-store");
            echo json_encode(array("error" => "Bad format of errors.xml"));
        }
        $this->clients = $aux;
    }
    
    public function getClients(){
        return $this->clients;
    }

}

?>
