<?php

/**
 * Description of OAuth
 * Configuration Class that gives an abstraction to the OAuth 2.0 assertion flow
 * @author Luis Javier Gomez Santana <luis.gomez at rediris.es>
 * @filesource
 */
include('OAuthClient.php');
include('LoadConfig.php');
include('LoadConfigJSON.php');

class OAuth {
    //constants that define the type of resource request to the server
    const HEADER = "HTTP_Authorization_Header";
    const GET = "URI_Query_Parameter";
    const BODY = "Form-Encoded_Body_Parameter";
    //constants that define the assertion type
    const SAML2 = "urn:oasis:names:tc:SAML:2.0:assertion";
    const PAPI = "urn:mace:rediris.es:papi";
    //constant that define the error response type
    const HTML = "HTML";
    const JSON = "JSON";

    //Error attributes
    protected $error;
    protected $error_type;
    protected $debug_active;
    //Client Credentials
    protected $client_id;
    protected $client_secret;
    //Assertion Types
    protected $assertion_type;
    //Access Token request Data
    protected $scope;
    protected $as;
    protected $grant_type;
    //Resource Token request Data
    protected $request_type;
    protected $resource;
    protected $rs;
    //LoadConfig
    protected $conf;
    //Added to separate the request of the access token from the request of the resource
    protected $oauth_client;
    //the type of the token local storage
    protected $local_storage;

    /**
     * Main constructor.
     * @param String $dir The path of the client configuration file     
     */
    public function __construct($dir="") {
        if ($dir == "") {
            $file = dirname(dirname(__FILE__)) . 'config/clientConfig.xml';
        } else {
            $file = $dir;
        }
        $this->error = null;
        $this->resource = null;
        //the conf file can be in two formats: XML and JSON, the client decide it when creates the OAuth object
        if (strcmp(substr($file, -3), 'xml') == 0) {
            $this->conf = new LoadConfig($file);
        } else {
            $this->conf = new LoadConfigJSON($file);
        }
        $this->debug_active = $this->conf->get_debug_active_client();
        $this->error_type = $this->conf->get_error_type();
        $this->client_id = $this->conf->get_client_id();
        $this->client_secret = $this->conf->get_client_secret();
        $this->scope = $this->conf->get_scope();
        $this->as = $this->conf->get_as();
        $this->grant_type = $this->conf->get_grant_type();
        $this->rs = $this->conf->get_rs();
        $this->assertion_type = $this->conf->get_assertion_type();
        $this->request_type = $this->conf->get_request_type();
        $this->oauth_client = new OAuthClient($this->client_id, $this->client_secret, $this->debug_active);
        $this->local_storage = $this->conf->get_storType();
    }

    /**
     * Put message in error log.
     * @param String $message The message to be inserted in the log
     */
    private function error($message) {
        if (strcmp($this->debug_active, true) == 0) {
            error_log("OAuth: " . $message);
        }
    }

    /**
     * Resquest an access token to Authorization Server
     * @param String $assertion The assertion safely obtained 
     */
    public function requestAccessToken($assertion) {
        $this->error('requestAccessTokenAS');
        if (strcmp($this->local_storage, "session") == 0) {
            $this->requestAccessTokenAS_session($assertion);
        } else if (strcmp($this->local_storage, "file") == 0) {
            $this->requestAccessTokenAS_file($assertion);
        } else if (strcmp($this->local_storage, "db") == 0) {
            $this->requestAccessTokenAS_db($assertion);
        } else
            $this->error = $this->returnError($this->oauth_client);
    }

    public function getIDSAMLAssertion($assertion) {
        $sx = new SimpleXMLElement($assertion, 0, false, 'urn:oasis:names:tc:SAML:2.0:assertion');
        $attr = (array) $sx->attributes();
        $id = $attr['@attributes']['ID'];
        //file_put_contents('/tmp/prueba.txt', print_r($id, true), FILE_APPEND | LOCK_EX);
        return $id;
    }

    /**
     * Resquest an access token to Authorization Server if there isn't another
     * one for that user saved in the session
     * @param String $assertion 
     */
    public function requestAccessTokenAS_session($assertion) {
        $this->error("requestAccessTokenAS_session");
        session_start();
        //session_destroy();
        if (strcmp($this->getAssertion_type(), "urn:mace:rediris.es:papi") == 0)
            $uid = $assertion['uid'];
        elseif (strcmp($this->getAssertion_type(), "urn:oasis:names:tc:SAML:2.0:assertion") == 0)
            $uid = $this->getIDSAMLAssertion($assertion);
        if (!(isset($_SESSION[$uid])) || $_SESSION[$uid]['exp_in'] < microtime(true) || !(isset($_SESSION[$uid]['token']))) {
            if (!$this->oauth_client->doAccessTokenRequest($this->as, $this->scope, $assertion, $this->assertion_type, $this->grant_type))
                $this->error = $this->returnError($this->oauth_client);
            $token = $this->oauth_client->getAccess_token();
            $time = microtime(true) + $this->oauth_client->getExpires_in();
            $_SESSION[$uid] = array();
            $_SESSION[$uid]['exp_in'] = $time;
            $_SESSION[$uid]['token'] = $token;
        } else {
            $this->oauth_client->setAccess_token($_SESSION[$uid]['token']);
        }
    }

    public function requestAccessTokenAS_file($assertion) {
        $this->error("requestAccessTokenAS_file");
        @include dirname(__FILE__) . '/oauth_client/client_token/token.php';
        if (!(isset($array_token[$assertion['uid']])) || $array_token[$assertion['uid']][0] < time()) {
            if (!$this->oauth_client->doAccessTokenRequest($this->as, $this->scope, $assertion, $this->assertion_type, $this->grant_type))
                $this->error = $this->returnError($this->oauth_client);
            if (strcmp($this->getAssertion_type(), "urn:mace:rediris.es:papi") == 0) {
                $token = $this->oauth_client->getAccess_token();
                $uid = $assertion['uid'];
                $time = microtime(true) + $this->oauth_client->getExpires_in();
                $cad = "<?php \$array_token['" . $uid . "'] = array(" . $time . ", '" . $token . "'); ?>";
                file_put_contents("oauth_client/client_token/token.php", $cad . PHP_EOL, FILE_APPEND | LOCK_EX);
            }
        } else {
            $this->oauth_client->setAccess_token($array_token[$assertion['uid']][1]);
        }
    }

    public function requestAccessTokenAS_db($assertion) {
        $this->error('requestAccessTokenAS_db');
        $dbh = dba_open('oauth_client/client_token/token_data.db', "c", "inifile") or die("Couldn't open database");
        //$uid = $assertion['uid'];
        if (strcmp($this->getAssertion_type(), "urn:mace:rediris.es:papi") == 0)
            $uid = $assertion['uid'];
        elseif (strcmp($this->getAssertion_type(), "urn:oasis:names:tc:SAML:2.0:assertion") == 0)
            $uid = $this->getIDSAMLAssertion($assertion);
        if (!(dba_exists($uid, $dbh))) {
            if (!$this->oauth_client->doAccessTokenRequest($this->as, $this->scope, $assertion, $this->assertion_type, $this->grant_type))
                $this->error = $this->returnError($this->oauth_client);
            $token = $this->oauth_client->getAccess_token();
            $time = time() + 15;
            $arr = json_encode(array('expIn' => $time, 'token' => $token));
            dba_insert($uid, $arr, $dbh) or die('Error inserting the token in DB');
        } else {
            $json_db = dba_fetch($uid, $dbh);
            $array_json = json_decode($json_db, true);
            $expIn = $array_json['expIn'];
            $token = $array_json['token'];
            if ($expIn < time()) {
                if (!$this->oauth_client->doAccessTokenRequest($this->as, $this->scope, $assertion, $this->assertion_type, $this->grant_type))
                    $this->error = $this->returnError($this->oauth_client);
                $token = $this->oauth_client->getAccess_token();
                $time = microtime(true) + $this->oauth_client->getExpires_in();
                $arr = json_encode(array('expIn' => $time, 'token' => $token));
                //dba_replace no reeemplaza, cambia nombre anterior y inserta nuevo
                //dba_delete($uid, $dbh);
                dba_replace($uid, $arr, $dbh) or die('Error inserting the token in DB');
            } else {
                //$db_json = dba_fetch($uid, $dbh);
                $this->oauth_client->setAccess_token($token);
            }
        }
        dba_close($dbh);
    }

    /**
     * Request the resource to the Resource Server
     * @param String $rs
     * @param String $request_type
     * @return boolean True if the resource was received succesfully, False otherwise 
     */
    public function requestResourceRS($rs, $request_type) {
        $this->error("requestResourceRS");
        $dev = false;
        if (!$this->oauth_client->requestResource($this->rs, $this->request_type, $this->as)) {
            $this->error = $this->returnError($this->oauth_client);
        } else {
            $this->resource = $this->returnResource($this->oauth_client);
            $dev = true;
        }
        return $dev;
    }

    /**
     *
     * @param $scope
     * @return type string 
     */
    private function cleanScope($scope) {
        if (strpos($scope, "?") == 0) {
            $res = $scope;
        } else {
            $res = substr($scope, 0, strpos($scope, "?"));
        }
        return $res;
    }

    /**
     * Returns the formatted resource
     * @param OAuthClient $oauth
     * @return  String The formatted response
     */
    public function returnResource($oauth) {
        $this->error("returnResource");
        if ($this->conf->hasFormatClass($this->cleanScope($this->scope))) {
            $class = $this->conf->getFormatClass($this->cleanScope($this->scope));
            if ($this->conf->hasFormatArchiveName($this->cleanScope($this->scope))) {
                include_once dirname(__FILE__) . "/response_formats/" . $this->conf->getFormatArchiveName($this->cleanScope($this->scope));
                $reflect = new ReflectionClass($class);
                $string = $reflect->newInstance()->formatResource($oauth->getResource());
            } else {
                $this->error("You must fill the clientConfig.xml file before getting the resource. FormatArchiveName missing.");
                $string = "<br/>You must fill the clientConfig.xml file before getting the resource.</br>";
            }
        } else {
            $this->error("Resource without formatting. Develop a FormattingResponse Class to improve the functionality.");
            $string = $oauth->getResource();
        }
        return $string;
    }

    /**
     *
     * @param OAuthClient $oauth
     * @return String The formatted error 
     */
    public function returnError($oauth) {
        $this->error("returnError");
        if (0 == strcmp($this->error_type, OAuth::HTML)) {
            return $oauth->getHTMLError();
        } else {
            return $oauth->getJSONError();
        }
    }

    //Setters of the Assertion type

    /**
     * Set the assertion type to a PAPI assertion
     */
    public function setPAPIAssertionType() {
        $this->error("setPAPIAssertionType");
        $this->assertion_type = OAuth::PAPI;
    }

    /**
     * Set the assertion type to a SAML2 assertion
     */
    public function setSAML2AssertionType() {
        $this->error("setSAML2AssertionType");
        $this->assertion_type = OAuth::SAML2;
    }

    //Setters of the error response

    /**
     * Set the Error Response to an HTML String
     */
    public function setHTMLErrorResponse() {
        $this->error("setHTMLErrorResponse");
        $this->error_type = OAuth::HTML;
    }

    /**
     * Set the Error Response to a JSON element
     */
    public function setJSONErrorResponse() {
        $this->error("setJSONErrorResponse");
        $this->error_type = OAuth::JSON;
    }

    // Setters of the Resource Request

    /**
     * Set the resource request type to a POST request.
     */
    public function setBODYResourceRequest() {
        $this->error("setBODYResourceRequest");
        $this->request_type = OAuth::BODY;
    }

    /**
     * Set the resource request type to a Authorization HEADER request.
     */
    public function setHEADERResourceRequest() {
        $this->error("setHEADERResourceRequest");
        $this->request_type = OAuth::HEADER;
    }

    /**
     * Set the resource request type to a GET request.
     */
    public function setGETResourceRequest() {
        $this->error("setGETResourceRequest");
        $this->request_type = OAuth::GET;
    }

    // SimpleGetters
    public function getAssertion_type() {
        return $this->assertion_type;
    }

    public function getScope() {
        return $this->scope;
    }

    public function getAs() {
        return $this->as;
    }

    public function getGrant_type() {
        return $this->grant_type;
    }

    public function getResource() {
        return $this->resource;
    }

    public function getRs() {
        return $this->rs;
    }

    public function getError() {
        return $this->error;
    }

    public function getError_type() {
        return $this->error_type;
    }

    public function getClient_id() {
        return $this->client_id;
    }

    public function getClient_secret() {
        return $this->client_secret;
    }

    public function getRequest_type() {
        return $this->request_type;
    }

    //Simple Setters
    public function setScope($scope) {
        $this->scope = $scope;
    }

    public function setAs($as) {
        $this->as = $as;
    }

    public function setGrant_type($grant_type) {
        $this->grant_type = $grant_type;
    }

    public function setRs($rs) {
        $this->rs = $rs;
    }

    public function getOAuth_client() {
        return $this->oauth_client;
    }

}

?>
